; Example Program xram01.a51
; Access External Data Memory using I/O Port
;
; Memory Allocation
SRAM_DATA	EQU	30h	; SRAM Data
UART_BUSY	EQU	00h	; UART Busy Flag
; Symbol Definition
SRAM_ADDR	EQU	0000h	; 0000 to 1FFF for 6264  ( 8Kx8)
;Pin Assignment
DATA_PORT	EQU	P0
LO_ADDR_PORT	EQU	P1
HI_ADDR_PORT	EQU	P2
SRAM_CS		EQU	P3.5
SRAM_WR		EQU	P3.6
SRAM_RD		EQU	P3.7
;
PROG	EQU	0000h
	ORG	PROG+0000h
	SJMP	START
;Interrupt Vector Table
	ORG	PROG+0003h
	LCALL	INT0_ISR
	RETI
	ORG	PROG+000Bh
	LCALL	T0_ISR
	RETI
	ORG	PROG+0013h
	LCALL	INT1_ISR
	RETI
	ORG	PROG+001Bh
	LCALL	T1_ISR
	RETI
	ORG	PROG+0023h
	LCALL	UART_ISR
	RETI
;
	ORG	PROG+0030h
START:
        MOV     PCON,#10000000b	; Double Baud Rate
        MOV     TMOD,#00100001b	; Timer1 in Mode 2, Timer0 in Mode 1
        MOV     TH1, #0FFh	; Baud Rate = 57600 bps at 11.0592MHz
        MOV     SCON,#01010000b	; UART in Mode 1
	SETB	ES		; Enable UART Interrupt
	SETB	EA		; Enable Interrupt
	SETB	TR1		; Start Timer 1
	CLR	UART_BUSY
; Write "ABC..." into SRAM
	MOV	DPTR,#SRAM_ADDR
	MOV	SRAM_DATA,#'A'
LOOP_1:	MOV	A,SRAM_DATA
	LCALL	SRAM_WRITE
	INC	SRAM_DATA
	MOV	A,SRAM_DATA
	CJNE	A,#'Z'+1,INC_SRAM_ADDR
	MOV	SRAM_DATA,#'A'
INC_SRAM_ADDR:
	INC	DPTR
	MOV	A,DPH
	CJNE	A,#20h,LOOP_1
; Read bytes from SRAM and sent to UART
	MOV	DPTR,#SRAM_ADDR
LOOP_2:	LCALL	SRAM_READ
	MOV	SRAM_DATA,A
WAIT_UART:
	JB	UART_BUSY,WAIT_UART
	SETB	UART_BUSY
	MOV	SBUF,SRAM_DATA
	INC	DPTR
	MOV	A,DPH
	CJNE	A,#20h,LOOP_2
DONE:	SJMP	DONE
;
INT0_ISR:
	RET
T0_ISR:
	RET
INT1_ISR:
	RET
T1_ISR:
	RET
UART_ISR:
        JB      RI,RECEIVED
TRANSMITTED:
	CLR	UART_BUSY
        CLR     TI
        RET
RECEIVED:
	CLR     RI
	RET
;
SRAM_READ:
	MOV	LO_ADDR_PORT,DPL
	MOV	HI_ADDR_PORT,DPH
	MOV	DATA_PORT,#0FFh
	CLR	SRAM_CS
	CLR	SRAM_RD
	MOV	A,DATA_PORT
	SETB	SRAM_RD
	SETB	SRAM_CS
	RET
SRAM_WRITE:
	MOV	LO_ADDR_PORT,DPL
	MOV	HI_ADDR_PORT,DPH
	CLR	SRAM_CS
	MOV	DATA_PORT,A
	CLR	SRAM_WR
	SETB	SRAM_WR
	SETB	SRAM_CS
	RET
;
	END